package com.example.micrometer;

import com.google.common.util.concurrent.ThreadFactoryBuilder;
import io.micrometer.core.instrument.MeterRegistry;
import io.micrometer.core.instrument.binder.jvm.ExecutorServiceMetrics;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.Primary;

import java.util.concurrent.ArrayBlockingQueue;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.ThreadPoolExecutor;

import static java.util.concurrent.TimeUnit.SECONDS;

@Configuration
public class ExecutorServiceConfig {
    @Bean
    @Primary
    public ExecutorService executorService(MeterRegistry meterRegistry) {
        return executorService(meterRegistry, "primary");

    }

    @Bean
    public ExecutorService ioExecutorService(MeterRegistry meterRegistry) {
        return executorService(meterRegistry, "io");
    }


    @Bean
    public ExecutorService genericExecutorService(MeterRegistry meterRegistry) {
        return executorService(meterRegistry, "generic");
    }

    public ExecutorService executorService(MeterRegistry meterRegistry, String name) {
        return ExecutorServiceMetrics.monitor(meterRegistry, new ThreadPoolExecutor(10, 100,
                30, SECONDS, new ArrayBlockingQueue<>(10000), new ThreadFactoryBuilder().setNameFormat(name + "-%s").build(),
                new ThreadPoolExecutor.CallerRunsPolicy()), name, name);
    }

}
